# controls.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Author: Brian Bernas
#
# Additional controls that are used by the interface

from __init__ import get_translation
from Tkinter import *

_ = get_translation('tk_interface')

#-------------------------------------------------------------------------------
class Dialog(Toplevel):
	def __init__(self, parent, title=None):
		self.__parent = parent
		self.__title = title
		self._validate = FALSE	# Value that refelcts whether the user clicked OK or Cancel
		self._resizable = FALSE

	def do_modal(self):
		# Display the dialog in modal mode
		parent = self.__parent
		Toplevel.__init__(self, parent)
		self.transient(parent)

		if self.__title:
			self.title(self.__title)

		body = Frame(self)
		self.initial_focus = self.body(body)
		body.pack(fill=BOTH, expand=TRUE, padx=5, pady=5)

		self.button_box()

		self.grab_set()

		if not self.initial_focus:
			self.initial_focus = self

		self.protocol('WM_DELETE_WINDOW', self.cancel)
		self.geometry("+%d+%d" %(parent.winfo_rootx()+50, parent.winfo_rooty()+50))
		self.initial_focus.focus_set()
		if not self._resizable:
			self.resizable(FALSE, FALSE)
		self.wait_window(self)

		return self._validate

	def destroy(self):
		self.initial_focus = None
		Toplevel.destroy(self)

	def body(self, master):
		"""This should be overriden to create the controls for the dialog."""
		pass

	def button_box(self):
		# Create the main dialog buttons
		frame = Frame(self)
		Button(frame, text=_("OK"), width=10, command=self.ok, default=ACTIVE).pack(
			side=LEFT, padx=5, pady=5)
		Button(frame, text=_("Cancel"), width=10, command=self.cancel).pack(side=LEFT, padx=5, pady=5)

		self.bind('<Return>', self.ok)
		self.bind('<Escape>', self.cancel)

		frame.pack(side=RIGHT)

	def ok(self, event=None):
		if self.apply():
			self.withdraw()
			self.update_idletasks()

			self._validate = TRUE
			self.cancel()

	def cancel(self, event=None):
		self.__parent.focus_set()	# Set the focus back to the parent window
		self.destroy()

	def apply(self):
		"""Proccess the data.
		This is called after OK is clicked and returns TRUE if all data is verified.
		This should be overriden"""
		return TRUE

#-------------------------------------------------------------------------------
class ThinButton(Label):
	def __init__(self, master, command=None, **kw):
		if not kw.has_key("relief"): kw["relief"] = RAISED
		if not kw.has_key("border") and not kw.has_key("bd"): kw["border"] = 2
		
		Label.__init__(self, master, kw)

		self.command = command

		# Bindings
		self.bind('<Button-1>', self.__OnMouseDown)
		self.bind('<Button1-ButtonRelease>', self.__OnMouseUp)

	def __OnMouseDown(self, event):
		if self['state'] != DISABLED:
			self['relief'] = SUNKEN

	def __OnMouseUp(self, event):
		if self['state'] != DISABLED:
			self['relief'] = RAISED
			if self.command:
				self.command()
