# interface.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Author: Brian Bernas
#
# The interface only. Just the stuff that makes it pretty and the functions for button clicks

from __init__ import *
from Tkinter import *
from tkFileDialog import askopenfilename, asksaveasfilename
import tkMessageBox
from objectlistbox import *
from controls import *
import os, sys
from tdm_calendar import Calendar
from optionsdialog import OptionsDlg
import tdmcalls

_ = get_translation('tk_interface')

CONF_SECTION = "TkInterface"

#-------------------------------------------------------------------------------
class Interface:
	def __init__(self, app, master):
		self._app = app
		self._master = master
		self._font = None
		self.__sorting_method = IntVar()
		self.__filter_method = IntVar()
		self.__current_group = None
		self.__disable_finish = IntVar()
		self.__view_calendar = IntVar()
		self.__calendar_filter = IntVar()
		self.__phantom_fields = FALSE
		self.__modified_name = FALSE
		self.__curdir = os.getcwd()
		self.__task_names = []
		self.__groups = []
		self.__sort_count = 0
		self.__sort_list = [None, "alphabetical", "priority"]
		self.__filter_count = 0
		self.__filter_list = [None, "show_unfinished"]
		self.__view_group_list = IntVar()

		master.title(appname)

		# Set the option file
		# General options file
		gen = os.path.join(tdmcalls.get_root_directory(), "tk_options")
		# User's options file
		user = os.path.join(tdmcalls.get_user_directory(), "tk_options")

		conf = None
		if os.path.isfile(user): conf = user
		elif os.path.isfile(gen): conf = gen

		if conf:
			master.option_readfile(conf)
		else:
			print _("The Tk Option file could not be found.")

		font = master.option_get("font", Tk)
		if font:
			self._font = tuple(font.split())
		else:
			# Default back to Arial 10pt
			self._font = ('Arial', 10)

		# Create the menu
		menu = Menu(master)
		master.config(menu=menu)

		# File menu
		file = Menu(menu, tearoff=FALSE)

		core_load_plugins = app.get_setting("Core", "loadplugins", 1)

		if core_load_plugins:
			self._import_menu = file.import_menu = Menu(menu, tearoff=FALSE)
			self._export_menu = file.export_menu = Menu(menu, tearoff=FALSE)

		file.add_command(label=_("New"), accelerator="Ctrl+N", command=self.OnNew)
		file.add_command(label=_("Open"), accelerator="Ctrl+O", command=self.OnOpen)
		file.add_command(label=_("Save"), accelerator="Ctrl+S", command=self.OnSave)
		file.add_command(label=_("Save As"), command=self.OnSaveAs)
		file.add_separator()
		file.add_command(label=_("Reload Default"), command=self.OnReloadDefault)

		if core_load_plugins:
			file.add_separator()
			file.add_cascade(label=_("Import"), menu=file.import_menu)
			file.add_cascade(label=_("Export"), menu=file.export_menu)

		file.add_separator()
		file.add_command(label=_("Options"), accelerator="Ctrl+P", command=self.OnOptions)
		file.add_separator()
		file.add_command(label=_("Exit"), accelerator="Alt+X", command=self.OnExit)

		# View menu
		view = Menu(menu, tearoff=FALSE)
		view.add_checkbutton(label=_("Show Calendar"), accelerator="Ctrl+D",
			variable=self.__view_calendar)
		view.add_checkbutton(label=_("Show Group List"), accelerator="Ctrl+G",
			variable=self.__view_group_list)
		view.add_separator()
		view.add_command(label=_("Refresh Task List"), command=self.OnSortingMethod)

		# Sorting menu
		self._sorting_menu = sorting = Menu(menu, tearoff=FALSE)
		sorting.add_radiobutton(label=_("None"), value=0, variable=self.__sorting_method)
		sorting.add_radiobutton(label=_("Alphabetical"), value=1, variable=self.__sorting_method)
		sorting.add_radiobutton(label=_("Priority"), value=2, variable=self.__sorting_method)

		if core_load_plugins:
			sorting.add_separator()

		self.__sort_count = 3

		# Filter menu
		self._filter_menu = filter = Menu(menu, tearoff=FALSE)
		filter.add_radiobutton(label=_("None"), value=0, variable=self.__filter_method)
		filter.add_radiobutton(label=_("Show Unfinished"), value=1, variable=self.__filter_method)

		if core_load_plugins:
			filter.add_separator()

		self.__filter_count = 2

		# Help menu
		help = Menu(menu, tearoff=FALSE)
		help.add_command(label=_("Help"), accelerator="F1", command=self.OnHelp)
		help.add_separator()
		help.add_command(label=_("About..."), command=self.OnHelpAbout)

		# Add the sub menus to the main menu
		menu.add_cascade(label=_("File"), menu=file)
		menu.add_cascade(label=_("View"), menu=view)
		menu.add_cascade(label=_("Sorting"), menu=sorting)
		menu.add_cascade(label=_("Filters"), menu=filter)
		menu.add_cascade(label=_("Help"), menu=help)

		# The main interface, this is the really fun part
		frame = Frame(master, bd=0)
		Label(frame, text=_("Items To Do:"), anchor=W).grid(row=0, column=1, sticky=NW)

		# Calendar and group list stuff
		self._extrasframe = Frame(frame, bd=0)
		self._calendar = Label(self._extrasframe, bd=0, width=0)
		self._calendar.grid()
		self._calFilter = None
		self._groupList = Label(self._extrasframe, bd=0, width=0)
		self._groupList.grid()
		self._groupScroll = None

		self._extrasframe.grid_rowconfigure(0, weight=1)
		self._extrasframe.grid_rowconfigure(1, weight=1)
		self._extrasframe.grid(row=1, column=0, sticky=NS)

		# Tasklist
		self._taskList = tl = ObjectListbox(frame, bd=0, background='white', relief=FLAT,
			width=330, height=110,
			itemcolor=app.get_setting(CONF_SECTION, "itembackground", 'gray90'),
			selectioncolor=app.get_setting(CONF_SECTION, "selecteditembackground", "#6A7F9E"))
		# Add the columns
		tl.insert_column(END, "", width=2, anchor=W,
			visible=app.get_setting(CONF_SECTION, "showpriority", TRUE))
		tl.insert_column(END, _("Task Name"), anchor=W, expand=TRUE)
		tl.insert_column(END, _("Due Date"), width=20,
			visible=app.get_setting(CONF_SECTION, "showduedate", FALSE))
		tl.insert_column(END, _("Finished"),
			visible=app.get_setting(CONF_SECTION, "showfinished", TRUE))
		tl.insert_column(END, _("Finished Time"), anchor=E, width=20,
			visible=app.get_setting(CONF_SECTION, "showfinishedtime", TRUE))
		tl.update_columns()
		tl.grid(row=1, column=1, sticky=NSEW)

		tl.set_item_leftclick(self.OnTaskLeftClick)
		tl.set_item_rightclick(self.OnTaskRightClick)

		frame.grid_rowconfigure(1, weight=1)
		frame.grid_columnconfigure(1, weight=1)
		frame.pack(side=TOP, fill=BOTH, expand=TRUE)

		# The control frame. Most interaction with tasks is done here
		frame = Frame(master, bd=2, relief=GROOVE)
		frame2 = Frame(frame, bd=0)

		Label(frame2, text=_("Task Name:")).grid(row=0, sticky=W)
		self._nameEntry = Entry(frame2, bd=2, relief=SUNKEN, width=35)
		self._nameEntry.grid(row=0, column=1, sticky=W)
		Label(frame2, text=_("Task Group:")).grid(row=1, sticky=W)
		self._groupEntry = Entry(frame2, bd=2, relief=SUNKEN, width=35)
		self._groupEntry.grid(row=1, column=1, sticky=W)
		frame2.grid(row=0, sticky=W)

		frame2 = Frame(frame, bd=0)
		Label(frame2, text=_("Due Date:")).grid(row=0, sticky=W)
		self._dueDate = Entry(frame2, bd=2, relief=SUNKEN, width=20, justify=CENTER)
		self._dueDate.grid(row=0, column=1, sticky=W)
		# Label(frame2, text=_("Occurance:")).grid(row=0, column=2, sticky=W)
		# self._taskOccurance = Entry(frame2, bd=2, relief=SUNKEN, width=15)
		# self._taskOccurance.grid(row=0, column=3, sticky=W)
		frame2.grid(row=1, sticky=W)

		frame2 = Frame(frame, bd=0)
		self._finished = ThinButton(frame2, text=_("Finished"), width=17, state=DISABLED,
			command=self.OnFinished)
		self._finished.grid(row=0, sticky=W)
		self._disableFinish = Checkbutton(frame2, text=_("Disable Finish"),
			variable=self.__disable_finish)
		self._disableFinish.grid(row=0, column=1, sticky=W)
		frame2.grid(row=2, sticky=W)

		Label(frame, text=_("Notes:")).grid(row=3, sticky=W)
		frame2 = Frame(frame, bd=0)
		self._notesText = Text(frame2, wrap=WORD, width=55, height=5)
		scroll = Scrollbar(frame2, command=self._notesText.yview)
		self._notesText.config(yscrollcommand=scroll.set)
		self._notesText.grid(row=0, sticky=EW)
		scroll.grid(row=0, column=1, sticky=NS)
		frame2.grid_columnconfigure(0, weight=1)
		frame2.grid(row=4, sticky=EW)

		frame2 = Frame(frame, bd=0)
		ThinButton(frame2, text=_("Clear Fields"), command=self.OnClearFields).grid(
			row=0, sticky=W)
		self._add = ThinButton(frame2, text=_("Add"), command=self.OnAdd)
		self._add.grid(row=0, column=1, padx=10, sticky=E)
		self._remove = ThinButton(frame2, text=_("Remove"), state=DISABLED, command=self.OnRemove)
		self._remove.grid(row=0, column=2, sticky=E)
		frame2.grid_columnconfigure(0, weight=1)
		frame2.grid(row=5, sticky=NSEW)

		# Grid settings
		frame.grid_columnconfigure(0, weight=1)
		frame.grid_rowconfigure(4, weight=1, minsize=20)
		frame.pack(side=LEFT, fill=X, expand=TRUE, anchor=W)

		# Accelerator key bindings
		master.bind('<Control-n>', self.OnNew); master.bind('<Control-N>', self.OnNew)
		master.bind('<Control-o>', self.OnOpen); master.bind('<Control-O>', self.OnOpen)
		master.bind('<Control-s>', self.OnSave); master.bind('<Control-S>', self.OnSave)
		master.bind('<Control-a>', self.OnAdd); master.bind('<Control-A>', self.OnAdd)
		master.bind('<Control-r>', self.OnRemove); master.bind('<Control-R>',self.OnRemove)
		master.bind('<Control-f>', self.OnFinished); master.bind('<Control-F>', self.OnFinished)
		master.bind('<Control-p>', self.OnOptions); master.bind('<Control-P>', self.OnOptions)
		master.bind('<Control-l>', self.OnClearFields)
		master.bind('<Control-L>', self.OnClearFields)
		master.bind('<Control-d>', lambda e, s=self:
			s.__view_calendar.set(not s.__view_calendar.get()))
		master.bind('<Control-D>', lambda e, s=self:
			s.__view_calendar.set(not s.__view_calendar.get()))
		master.bind('<Control-g>', lambda e, s=self:
			s.__view_group_list.set(not s.__view_group_list.get()))
		master.bind('<Control-G>', lambda e, s=self:
			s.__view_group_list.set(not s.__view_group_list.get()))

		master.bind('<F1>', self.OnHelp)
		self._nameEntry.bind('<Return>', self.OnAdd)
		self._nameEntry.bind('<Tab>', lambda e, s=self: s.phantom_fields(0))
		self._nameEntry.bind('<KeyPress>', lambda e, s=self: s.phantom_fields(-1))
		self._groupEntry.bind('<Return>', self.OnAdd)
		self._groupEntry.bind('<KeyPress>', lambda e, s=self: s.phantom_fields(0))
		self._dueDate.bind('<Return>', self.OnAdd)
		self._dueDate.bind('<FocusIn>', self.OnDueDateFocus)
		self._dueDate.bind('<KeyPress>', lambda e, s=self: s.phantom_fields(0))
		self._notesText.bind('<Control-Return>', self.OnAdd)
		self._notesText.bind('<KeyPress>', lambda e, s=self: s.phantom_fields(0))
		master.bind('<Alt-F4>', self.OnExit)
		master.bind('<Alt-x>', self.OnExit); master.bind('<Alt-X>', self.OnExit)

		master.protocol('WM_DELETE_WINDOW', self.OnExit)

		self.__disable_finish.trace('w', self.OnDisableFinish)
		self.__view_calendar.trace('w', self.show_calendar)
		self.__calendar_filter.trace('w', self.OnCalendarFilter)
		self.__view_group_list.trace('w', self.show_group_list)

		self.__sorting_method.trace('w', self.OnSortingMethod)
		self.__filter_method.trace('w', self.OnSortingMethod)

		# The priority dropdown window. This is tricky
		self.__priority_dropdown = dd = Toplevel(master, bd=0)
		dd.withdraw()
		dd.overrideredirect(1)

		dd.bind('<FocusOut>', lambda e, d=dd: d.withdraw())

		canvas = Canvas(dd, bd=1, relief=SOLID, highlightthickness=0, width=11, height=70)
		canvas.pack(side=TOP)

		i = 1
		for c in ["#FFDDDD", "#FFBBBB", "#FF5B5B", "#E70000", "#9A0000"]:
			tag = "p%d" %i
			fill = ['black', 'white'][i > 3]

			canvas.create_rectangle(1, ((i-1)*14)+1, 11, ((i-1)*14)+14, fill=c, tag=tag)
			canvas.create_text(6, ((i-1)*14)+8, text=`i`, fill=fill, font=("Arial", '10', 'bold'),
				tag=tag)

			canvas.tag_bind(tag, '<Button-1>', lambda e, s=self, i=i: s.OnClickPriorityDropdown(i))

			i = i + 1


		self._taskList.update()	# So the controls are fully drawn

	def show_group_list(self, *args):
		self._groupList.destroy()
		if self._groupScroll:
			self._groupScroll.destroy()

		if self.__view_group_list.get():
			self._groupList = Listbox(self._extrasframe, bd=2, relief=SUNKEN)
			self._groupScroll = Scrollbar(self._extrasframe, command=self._groupList.yview)
			self._groupList.config(yscrollcommand=self._groupScroll.set)
			self._groupList.grid(row=0, column=1, rowspan=2, sticky=NS)
			self._groupScroll.grid(row=0, column=2, rowspan=2, sticky=NS)

			self._groupList.bind('<ButtonRelease-1>', lambda e, s=self: s.OnSortingMethod())

			self.update_groups()

		else:
			self.__current_group = None
			self._groupList = Label(self._extrasframe, bd=0, width=0)
			self._groupList.grid()
			self._groupScroll = Label(self._extrasframe, bd=0, width=0)
			self._groupScroll.grid()
			self.OnSortingMethod()

	def show_calendar(self, *args):
		# Delete the existing calendar object
		self._calendar.destroy()
		if self._calFilter:
			self._calFilter.destroy()

		if self.__view_calendar.get():
			self._calendar = Calendar(self._extrasframe, bd=2, relief=SUNKEN, bg='white',
				command=self.OnCalendarSelectDate)
			self._calendar.grid(row=0, sticky=NW)

			# Calendar filter checkbutton
			self._calFilter = Checkbutton(self._extrasframe,
				text=_("Only show tasks due on\nthe selected date."), justify=LEFT,
				variable=self.__calendar_filter)
			self._calFilter.grid(row=1, sticky=NW)
		else:
			# A blank label to force self._extrasframe to shrink down after the
			# calendar has been removed
			self._calendar = Label(self._extrasframe, bd=0, width=0)
			self._calendar.grid(row=0)
			self._calFilter = Label(self._extrasframe, bd=0, width=0)
			self._calFilter.grid(row=1)

		# If the calendar filter is active, then redraw the tasks
		if self.__calendar_filter.get():
			self.update_list()

	def OnCalendarFilter(self, *args):
		self.update_list()

	def OnCalendarSelectDate(self, date):
		if self.__calendar_filter.get():
			self.update_list()

	def PrepNewFile(self):
		# Everything that should be done before a new file is loaded
		self._taskList.selection_set(-1)
		self._remove['state'] = DISABLED
		self._finished['state'] = DISABLED
		self.OnClearFields()

	def OnNew(self, event=None):
		self.PrepNewFile()
		self._app.new_file(ui=self)

	def OnOpen(self, event=None):
		self.PrepNewFile()
		self._app.do_file_open(ui=self)
		self.OnSortingMethod()

	def OnSave(self, event=None):
		self._app.do_file_save(ui=self)

	def OnSaveAs(self, event=None):
		self._app.do_file_saveas(ui=self)

	def OnReloadDefault(self):
		self.PrepNewFile()
		self._app.load_default_file(ui=self)
		self.OnSortingMethod()

	def OnOptions(self, event=None):
		app = self._app
		dlg = OptionsDlg(self._master, _("Options"))

		dlg.set_option("startupfile", app.get_setting("Core", "startupfile", 0))
		dlg.set_option("webbrowser", app.get_setting("Core", "webbrowser", ''))
		dlg.set_option("loadplugins", app.get_setting("Core", "loadplugins", 2))
		dlg.set_option("plugin_info", app.get_plugin_info())
		dlg.set_option("timeformat", app.get_setting("Core", "timeformat", ''))
		dlg.set_option("savebackup", app.get_setting("Core", "savebackup", FALSE))

		dlg.set_option("savewinposition", app.get_setting(CONF_SECTION, "savewinposition", TRUE))
		dlg.set_option("savewinsize", app.get_setting(CONF_SECTION, "savewinsize", TRUE))
		dlg.set_option("sortandfilter", app.get_setting(CONF_SECTION, "sortandfilter", FALSE))
		dlg.set_option("showpriority", app.get_setting(CONF_SECTION, "showpriority", TRUE))
		dlg.set_option("showduedate", app.get_setting(CONF_SECTION, "showduedate", FALSE))
		dlg.set_option("showfinished", app.get_setting(CONF_SECTION, "showfinished", TRUE))
		dlg.set_option("showfinishedtime", app.get_setting(CONF_SECTION, "showfinishedtime", TRUE))
		dlg.set_option("showcalendar", app.get_setting(CONF_SECTION, "showcalendar", FALSE))
		dlg.set_option("showgroups", app.get_setting(CONF_SECTION, "showgroups", TRUE))

		dlg.set_option("windowxoffset", app.get_setting(CONF_SECTION, "windowxoffset", 0))
		dlg.set_option("windowyoffset", app.get_setting(CONF_SECTION, "windowyoffset", 0))

		# Color settings
		dlg.set_option("itemforeground", app.get_setting(CONF_SECTION, "itemforeground", 'black'))
		dlg.set_option("itembackground", app.get_setting(CONF_SECTION, "itembackground", 'gray90'))
		dlg.set_option("selecteditemforeground",
			app.get_setting(CONF_SECTION, "selecteditemforeground", 'white'))
		dlg.set_option("selecteditembackground",
			app.get_setting(CONF_SECTION, "selecteditembackground", "#6A7F9E"))
		dlg.set_option("dueitemforeground",
			app.get_setting(CONF_SECTION, "dueitemforeground", 'black'))
		dlg.set_option("selecteddueitemforeground",
			app.get_setting(CONF_SECTION, "selecteddueitemforeground", 'white'))
		dlg.set_option("overdueitemforeground",
			app.get_setting(CONF_SECTION, "overdueitemforeground", 'darkred'))
		dlg.set_option("selectedoverdueitemforeground",
			app.get_setting(CONF_SECTION, "selectedoverdueitemforeground", 'yellow'))

		if dlg.do_modal():
			app.save_setting("Core", "startupfile", dlg.get_option("startupfile", 0))
			app.save_setting("Core", "webbrowser", dlg.get_option("webbrowser", ''))
			app.save_setting("Core", "loadplugins", dlg.get_option("loadplugins", 2))
			app.save_setting("Core", "timeformat", dlg.get_option("timeformat", ''))
			app.save_setting("Core", "savebackup", dlg.get_option("savebackup", FALSE))

			app.save_setting(CONF_SECTION, "savewinposition", dlg.get_option("savewinposition", 0))
			app.save_setting(CONF_SECTION, "savewinsize", dlg.get_option("savewinsize", 0))
			app.save_setting(CONF_SECTION, "sortandfilter", dlg.get_option("sortandfilter", 0))
			app.save_setting(CONF_SECTION, "showpriority", dlg.get_option("showpriority", TRUE))
			app.save_setting(CONF_SECTION, "showduedate", dlg.get_option("showduedate", FALSE))
			app.save_setting(CONF_SECTION, "showfinished", dlg.get_option("showfinished", TRUE))
			app.save_setting(CONF_SECTION, "showfinishedtime",
				dlg.get_option("showfinishedtime", TRUE))
			app.save_setting(CONF_SECTION, "showcalendar", dlg.get_option("showcalendar", FALSE))
			app.save_setting(CONF_SECTION, "showgroups", dlg.get_option("showgroups", TRUE))

			app.save_setting(CONF_SECTION, "itemforeground", dlg.get_option("itemforeground"))
			app.save_setting(CONF_SECTION, "itembackground", dlg.get_option("itembackground"))
			app.save_setting(CONF_SECTION, "selecteditemforeground",
				dlg.get_option("selecteditemforeground"))
			app.save_setting(CONF_SECTION, "selecteditembackground",
				dlg.get_option("selecteditembackground"))
			app.save_setting(CONF_SECTION, "dueitemforeground",
				dlg.get_option("dueitemforeground"))
			app.save_setting(CONF_SECTION, "selecteddueitemforeground",
				dlg.get_option("selecteddueitemforeground"))
			app.save_setting(CONF_SECTION, "overdueitemforeground",
				dlg.get_option("overdueitemforeground"))
			app.save_setting(CONF_SECTION, "selectedoverdueitemforeground",
				dlg.get_option("selectedoverdueitemforeground"))

			if sys.platform[:3] != "win":
				app.save_setting(CONF_SECTION, "windowxoffset", dlg.get_option("windowxoffset", 0))
				app.save_setting(CONF_SECTION, "windowyoffset", dlg.get_option("windowyoffset", 0))

			tasklist = self._taskList
			# Set the item colors for the listbox
			tasklist['itemcolor'] = dlg.get_option("itembackground")
			tasklist['selectioncolor'] = dlg.get_option("selecteditembackground")

			# If any listbox columns have been turned visible/invisible
			tasklist.show_column(0, dlg.get_option("showpriority", TRUE))
			tasklist.show_column(2, dlg.get_option("showduedate", FALSE))
			tasklist.show_column(3, dlg.get_option("showfinished", TRUE))
			tasklist.show_column(4, dlg.get_option("showfinishedtime", TRUE))

			tasklist.update_columns()
			tasklist.update()
			self.update_list()

	def OnHelp(self, event=None):
		self._app.show_html_help(ui=self)

	def OnHelpAbout(self):
		dlg = AboutDlg(self._master, _("About %s") %appname)
		dlg.about_text = self._app.get_about_text()
		dlg.home_page_command = self._app.show_home_page
		dlg.do_modal()

	def OnAdd(self, event=None):
		tasklist = self._taskList
		app = self._app

		name = self._nameEntry.get().strip()
		if name:
			tasklist.start_draw_cache()
			app.add_task(name)
			app.set_task_value(name, "Group", self._groupEntry.get().strip(), ui=self)
			app.set_task_value(name, "DueDate", self._dueDate.get().strip(), ui=self)
			app.set_task_value(name, "Notes", self._notesText.get('1.0', END).rstrip(), ui=self)
			app.set_task_value(name, "DisableFinish", self.__disable_finish.get(), ui=self)
			self.OnSortingMethod()
			tasklist.end_draw_cache()

			# The core formats the due date, this keeps the entry field up to date
			self._dueDate.delete(0, END)
			self._dueDate.insert(0, app.get_task_value(name, "DueDate"))
			self._groupEntry.delete(0, END)
			self._groupEntry.insert(0, app.get_task_value(name, "Group"))

			self._nameEntry.focus_set()
			self.__modified_name = FALSE
			self.phantom_fields()
			self._nameEntry.selection_range(0, END)

	def OnRemove(self, event=None):
		sel = self._taskList.curselection()
		name = self._taskList.get_column_text(sel, 1)
		if name:
			self._app.delete_task(name)
			self.OnClearFields()
			self._taskList.selection_set(-1)

	def OnDueDateFocus(self, event):
		if self._nameEntry.get() and not event.widget.get():
			# Get the current date and time
			# If the calendar is visible, get the date from it.
			if self.__view_calendar.get():
				y, m, d = self._calendar.GetSelDate()
				t = (y, m, d)
			else:
				# Eliminate time info
				t = tdmcalls.curtime()[:3]

			self.phantom_fields(0)
			event.widget.insert(0, self._app.fmttime(t))
			event.widget.selection_range(0, END)
			event.widget.focus_set()

	def OnDisableFinish(self, a, b, c):
		self.phantom_fields(0)
		if self._taskList.curselection() == -1:
			return

		if self.__disable_finish.get():
			self._dueDate.delete(0, END)
			self._dueDate['state'] = DISABLED
			self._finished['text'] = _("Finished")
			self._finished['state'] = DISABLED
		else:
			self._dueDate['state'] = NORMAL
			self._finished['state'] = NORMAL

	def OnFinished(self, event=None):
		cursel = self._taskList.curselection()
		name = self._taskList.get_column_text(cursel, 1)
		finished = self._app.get_task_value(name, "Finished")

		self._app.set_task_value(name, "Finished", not finished, ui=self)
		
		ret = self._app.get_task_value(name, "Finished")
		self._finished['text'] = _((N_("Finished"), N_("Not Finished"))[ret])
		self.OnSortingMethod()

	def OnClearFields(self, event=None):
		self._nameEntry.delete(0, END)
		self._groupEntry.delete(0, END)
		self._dueDate.delete(0, END)
		self._notesText.delete('1.0', END)
		self.__disable_finish.set(FALSE)
		self._nameEntry.focus_set()
		self._add['text'] = _("Add")
		self._finished['text'] = _("Finished")
		self.__modified_name = FALSE

	def OnDisplayPriorityDropdown(self, event=None):
		dd = self.__priority_dropdown
		dd.withdraw()	# Incase it's already displayed
		dd.focus_set()

		# Get the position of the parent window
		px = self._taskList.winfo_rootx()
		py = self._taskList.winfo_pointery()

		dd.geometry("+%d+%d" %(px, py))
		dd.deiconify()
		dd.tkraise()

	def OnClickPriorityDropdown(self, index):
		cursel = self._taskList.curselection()
		name = self._taskList.get_column_text(cursel, 1)
		task = self._app.set_task_value(name, "Priority", index, ui=self)
		self.__priority_dropdown.withdraw()
		self.OnSortingMethod()

	def OnTaskLeftClick(self, index):
		app = self._app

		self._remove['state'] = NORMAL

		name = self._taskList.get_column_text(index, 1)
		if not name: return

		self._nameEntry.delete(0, END)
		self._nameEntry.insert(0, name)
		self._nameEntry.selection_range(0, END)
		self._groupEntry.delete(0, END)
		self._groupEntry.insert(0, app.get_task_value(name, "Group"))
		self.__disable_finish.set(app.get_task_value(name, "DisableFinish"))
		self._dueDate.delete(0, END)
		self._dueDate.insert(0, app.get_task_value(name, "DueDate"))
		self._notesText.delete('1.0', END)
		self._notesText.insert('1.0', app.get_task_value(name, "Notes"))

		self._finished['text'] = _((N_("Finished"), N_("Not Finished"))[app.get_task_value(
			name, "Finished")])
		self._nameEntry.focus_set()

		self.phantom_fields(0)
		self.__phantom_fields = TRUE

	def OnTaskRightClick(self, index):
		pass

	def OnSortingMethod(self, *args):
		sorting_method = self.__sorting_method.get()
		filter_method = self.__filter_method.get()

		# Get the group that is currently active
		if self.__view_group_list.get():
			i = self._groupList.curselection()
			if len(i) > 0:
				self.__current_group = self._groupList.get(int(i[0]))

		# The sorting and filter settings must be in the range of the available values
		# If they aren't, set them to 0
		if sorting_method  < len(self.__sort_list):
			s = self.__sort_list[sorting_method]
		else:
			self.__sorting_method.set(0)
			s = 0

		if filter_method < len(self.__filter_list):
			f = self.__filter_list[filter_method]
		else:
			self.__filter_method.set(0)
			f = 0

		self._app.set_ui_sorting_filter(self, s, f, self.__current_group)

	def OnExit(self, event=None):
		# Tell the core that the interface is no longer in use
		if self._app.unbind_interface(self):
			# Destroy the main window
			self._master.quit()

	def draw_priority_box(self, canvas, row, column, x, y, tag):
		# Priority colors
		pc = ("#377B1F", "#FFDDDD", "#FFBBBB", "#FF5B5B", "#E70000", "#9A0000")

		# Get the task
		name = self._taskList.get_column_text(row, 1)
		if name:
			priority = self._app.get_task_value(name, "Priority")
			finished = self._app.get_task_value(name, "Finished")
		else:
			priority = 1
			finished = FALSE

		fg = ('black', 'white')[priority > 3]
		ptag = "p%d" %row

		# If a task is finished, show the box as green with a value of 'F"
		if finished:
			priority = 0
			fg = 'white'

		canvas.create_rectangle(x, y+1, x+10, y+16, fill=pc[priority], width=0, tags=(tag, ptag))

		if finished: priority = 'F'
		canvas.create_text(x+5, y+9, text=str(priority), fill=fg, font=("Arial", '10', 'bold'),
			tags=(tag, ptag))

		# Tag bindings
		if not finished:
			canvas.tag_bind(ptag, '<Button-1>', self.OnDisplayPriorityDropdown)

	def draw_finished_box(self, canvas, row, column, x, y, tag):
		name = self._taskList.get_column_text(row, 1)
		if self._app.get_task_value(name, "DisableFinish"): return

		ftag = "f%d" %row

		canvas.create_rectangle(x-6, y+4, x+6, y+15, fill="white", width=1, tags=(tag, ftag))
		if self._app.get_task_value(name, "Finished"):
			canvas.create_polygon(x-11, y+7, x-14, y+8, x, y+16, x+18, y+2, x+17, y, x, y+10,
				fill='darkblue', tags=(tag, ftag))

		canvas.tag_bind(ftag, '<Button-1>', self.OnFinished)

	def call(self, command, args):
		# These are small commands that external modules may want to call
		# args isn't a full argument list (*args) because the core will send a tuple
		if command == "set_current_filename":
			if not args[0] or (args[0] == 'DEFAULT'): val = appname
			else: val = "%s - %s" %(appname, args[0])
			self._master.title(val)
		elif command == "display_error": tkMessageBox.showerror(args[0], args[1])
		elif command == "display_warning": tkMessageBox.showwarning(args[0], args[1])
		elif command == "file_open_prompt":
			return askopenfilename(filetypes=[(args[0], args[1])], initialdir=self.__curdir)
		elif command == "file_saveas_prompt":
			return asksaveasfilename(filetypes=[(args[0], args[1])], initialdir=self.__curdir)
		elif command == "ask_yes_no":
			m = tkMessageBox.askquestion(args[0], args[1])
			return {'yes': 1, 'no': 0}[m]
		elif command == "ask_yes_no_cancel":
			m = tkMessageBox.Message(title=args[0], message=args[1], icon=tkMessageBox.QUESTION,
				type=tkMessageBox.YESNOCANCEL, master=self._master)
			sel = m.show()
			return {"yes": 1, "no": 0, "cancel": -1}[sel]

		# Listbox commands
		elif command == "insert_list_item":
			self.__task_names.append(args[0])
			tasklist = self._taskList
			i = tasklist.count()
			tasklist.insert_object(i, 0, self.draw_priority_box)
			tasklist.insert_text(i, 1, args[0])
			tasklist.insert_object(i, 3, self.draw_finished_box)
		elif command == "set_list_item_value":
			# Get the task format
			col1, col2, font = self.get_task_format(args[0])

			try:
				col = {
					"priority": 0,
					"name": 1,
					"duedate": 2,
					"finished": 3,
					"disablefinish": 3, # The value is 3 because it effects the finish value
					"finisheddate": 4,
					}[args[1]]
			except KeyError:
				# The value that was passed is not supported
				return

			row = self.get_listitem_index(args[0])
			if args[1] in ("name", "duedate", "finisheddate"):
				if args[1] == "duedate":
					# Redraw the name too
					self._taskList.insert_text(row, 1, args[0], color=col1, selectcolor=col2,
						font=font)

				self._taskList.insert_text(row, col, args[2], color=col1, selectcolor=col2,
					font=font)
			else:
				if args[1] in ("finished", "disablefinish"):
					self._taskList.insert_text(row, 1, args[0], color=col1, selectcolor=col2,
						font=font)
					self._taskList.insert_text(row, 2, self._app.get_task_value(args[0], "DueDate"),
						color=col1, selectcolor=col2, font=font)

				func = {
					"priority": self.draw_priority_box,
					"finished": self.draw_finished_box,
					"disablefinish": self.draw_finished_box,
					}[args[1]]
				self._taskList.insert_object(row, col, func)

		elif command == "remove_list_item":
			try:
				self.__task_names.remove(args[0])
				i = self.get_listitem_index(args[0])
				self._taskList.delete_item(i)
			except ValueError:
				pass

		elif command == "update_list_items":
			self.__task_names = args[0] # Holds the tasks that should be displayed
			self.update_list()
		elif command == "clear_list_items":
			self.__task_names = []
			self._taskList.delete_item(ALL)
		elif command == "update_list_groups":
			self.__groups = args[0]
			self.update_groups()

		# Plugins
		elif command == "add_plugin":
			p = args[0]
			if p['type'] == "file_import":
				self._import_menu.add_command(label=p['name'], command=lambda p=p, s=self:
					s._app.exec_plugin(p['value'], s))
			elif p['type'] == "file_export":
				self._export_menu.add_command(label=p['name'], command=lambda p=p, s=self:
					s._app.exec_plugin(p['value'], s, tasks=s.get_listitem_names()))
			elif p['type'] == "task_sorting":
				self._sorting_menu.add_radiobutton(label=p['name'], value=self.__sort_count,
					variable=self.__sorting_method)
				self.__sort_count = self.__sort_count + 1
				self.__sort_list.append(p['name'])

			elif p['type'] == "task_filter":
				self._filter_menu.add_radiobutton(label=p['name'], value=self.__filter_count,
					variable=self.__filter_method)
				self.__filter_count = self.__filter_count + 1
				self.__filter_list.append(p['name'])
		elif command == "remove_plugin":
			type = args[0]
			name = args[1]

			if type == "file_import": menu = self._import_menu
			elif type == "file_export": menu = self._export_menu
			elif type == "task_sorting": menu = self._sorting_menu
			elif type == "task_filter": menu = self._filter_menu
			else:
				return

			self.remove_plugin_menu_entry(menu, name)

		elif command == "save_settings":
			self.save_config()

	def remove_plugin_menu_entry(self, menu, name):
		# The menu must be a plugin menu
		if not menu in (self._import_menu, self._export_menu, self._sorting_menu,
			self._filter_menu):
				return FALSE

		old = tmp = None
		i = 0

		while 1:
			old = tmp
			tmp = menu.entrycget(i, 'label')
			if tmp == old: break # End of menu items
			elif name == tmp:
				menu.delete(i)
				return TRUE

			i = i + 1

		return FALSE

	def get_task_format(self, name):
		"""Return the formating for the specified task"""

		app = self._app
		font = self._font

		# The standard task colors
		fg = app.get_setting(CONF_SECTION, "itemforeground", 'black')
		selfg = app.get_setting(CONF_SECTION, "selecteditemforeground", 'white')
		duefg = app.get_setting(CONF_SECTION, "dueitemforeground", 'black')
		selduefg = app.get_setting(CONF_SECTION, "selecteddueitemforeground", 'white')
		overduefg = app.get_setting(CONF_SECTION, "overdueitemforeground", 'darkred')
		seloverduefg = app.get_setting(CONF_SECTION, "selectedoverdueitemforeground", 'yellow')

		duefont = None
		# Make sure that the font is valid
		if font:
			duefont = (font[0], font[1], "bold")

		task_due = app.is_task_due(name)

		# The task is due today
		if (task_due[0] == 0) and (task_due[1] <= 0):
			col1 = duefg; col2 = selduefg
			f = duefont
		# It's overdue
		elif (task_due[0] == 1) or (task_due[1] == 1):
			col1 = overduefg; col2 = seloverduefg
			f = font
		# Standard colors
		else:
			col1 = fg; col2 = selfg
			f = font

		# Return the task color, selected task color, and task font
		return (col1, col2, f)

	def update_list(self):
		tasklist = self._taskList
		app = self._app

		# See if the calendar filter is on
		calfilter = self.__calendar_filter.get()
		viewcalendar = self.__view_calendar.get()

		if viewcalendar:
			caldate = self._calendar.GetSelDate()

		# The list shouldn't update until the entire list is added
		tasklist.start_draw_cache()

		# Clear the list
		tasklist.delete_item(ALL)

		i = 0
		for name in self.__task_names:
			if calfilter and viewcalendar:
				duedate = app.get_task_value(name, "DueDate")
				fmt = app.get_setting("Core", "timeformat", "%Y-%m-%d")
				duedate = tdmcalls.strptime(fmt, duedate)


				# If it doesn't have a due date, then don't show it
				if not duedate: continue

				if tdmcalls.cmptime(caldate, duedate[:3]): continue

			col1, col2, font = self.get_task_format(name)

			tasklist.insert_object(i, 0, self.draw_priority_box)
			tasklist.insert_text(i, 1, name, color=col1, selectcolor=col2, font=font)
			tasklist.insert_text(i, 2, app.get_task_value(name, "DueDate"), color=col1,
				selectcolor=col2, font=font)
			tasklist.insert_object(i, 3, self.draw_finished_box)
			tasklist.insert_text(i, 4, app.get_task_value(name, "FinishedDate"), color=col1,
				selectcolor=col2, font=font)

			i = i + 1

		# End the draw caching so the list can update
		tasklist.end_draw_cache()

	def update_groups(self):
		if self.__view_group_list.get():
			self._groupList.delete(0, END)

			self._groupList.insert(0, _('ALL'))

			for g in self.__groups:
				self._groupList.insert(END, g)

	def get_listitem_index(self, name):
		tasklist = self._taskList
		for c in range(tasklist.count()):
			if tasklist.get_column_text(c, 1) == name:
				return c

	def get_listitem_names(self):
		tasklist = self._taskList
		names = []

		for i in range(tasklist.count()):
			names.append(tasklist.get_column_text(i, 1))

		return names

	def load_config(self):
		app = self._app

		if app.get_setting(CONF_SECTION, "savewinposition", TRUE):
			x = app.get_setting(CONF_SECTION, "mainwinxposition", 40) -\
				app.get_setting(CONF_SECTION, "windowxoffset", 0)
			y = app.get_setting(CONF_SECTION, "mainwinyposition", 40) -\
				app.get_setting(CONF_SECTION, "windowyoffset", 0)
			self._master.geometry("+%d+%d" %(x, y))

		if app.get_setting(CONF_SECTION, "savewinsize", TRUE):
			width = app.get_setting(CONF_SECTION, "mainwinwidth", 480)
			height = app.get_setting(CONF_SECTION, "mainwinheight", 320)
			self._master.geometry("%dx%d" %(width, height))

		if app.get_setting(CONF_SECTION, "showcalendar", FALSE):
			self.__view_calendar.set(TRUE)

		if app.get_setting(CONF_SECTION, "showgroups", TRUE):
			self.__view_group_list.set(TRUE)

		# Load the sorting and filter settings
		sf = str(app.get_setting(CONF_SECTION, "sortandfilter", ''))
		# Check for valid values
		if (',' in sf) and (len(sf) >= 3):
			s, f = sf.split(',')
			self.__sorting_method.set(int(s))
			self.__filter_method.set(int(f))

			# Change this back to an integer so everything else works peachy
			app.save_setting(CONF_SECTION, "sortandfilter", TRUE)

	def save_config(self):
		app = self._app

		if app.get_setting(CONF_SECTION, "savewinposition", TRUE):
			app.save_setting(CONF_SECTION, "mainwinxposition", self._master.winfo_x())
			app.save_setting(CONF_SECTION, "mainwinyposition", self._master.winfo_y())
		if app.get_setting(CONF_SECTION, "savewinsize", TRUE):
			app.save_setting(CONF_SECTION, "mainwinwidth", self._master.winfo_width())
			app.save_setting(CONF_SECTION, "mainwinheight", self._master.winfo_height())

		# If 'sortandfilter' then save them in the format 'sortandfilter = sort,filter'
		if app.get_setting(CONF_SECTION, "sortandfilter", FALSE):
			app.save_setting(CONF_SECTION, "sortandfilter", "%d,%d"
				%(self.__sorting_method.get(), self.__filter_method.get()))

	def phantom_fields(self, value=1):
		if value == 1:	# All data will be cleared when the task name changes
			self.__phantom_fields = TRUE
			self._groupEntry['fg'] = 'gray60'
			self._dueDate['fg'] = 'gray60'
			self._notesText['fg'] = 'gray60'
		elif value == 0:	# Disabled
			self.__phantom_fields = FALSE
			self._groupEntry['fg'] = 'black'
			self._dueDate['fg'] = 'black'
			self._notesText['fg'] = 'black'
			if not self.__modified_name:
				self._add['text'] = _("Apply")
		elif value == -1:	# All data is deleted
			self.__modified_name = TRUE
			self._add['text'] = _("Add")
			if self.__phantom_fields:
				self._groupEntry.delete(0, END)
				self._dueDate.delete(0, END)
				self._notesText.delete('1.0', END)
				self.__disable_finish.set(FALSE)

	def post_init(self):
		"""Actions that are executed after initialization"""

		self.load_config()
		self.OnSortingMethod()

#-------------------------------------------------------------------------------
class AboutDlg(Dialog):
	def __init__(self, parent, title=None):
		Dialog.__init__(self, parent, title)

		self.about_text = ''
		self.home_page_command = None

	def body(self, master):
		# Creating the controls
		Label(master, text=appname, font=("courier new", 16, 'bold'), fg='darkred').grid(row=0)
		Label(master, text=version, font=("courier new", 12, 'italic'), fg='darkblue').grid(
			row=0, column=1, sticky=SW)
		Button(master, text="http://todo-manager.sourceforge.net", fg='blue',
			activeforeground='blue', bd=0, cursor='hand2', command=self.__OnHomePage).grid(
			row=3, columnspan=2, sticky=W)
		frame = Frame(master, bd=2, relief=SUNKEN)
		text = Text(frame, bd=0, relief=FLAT, width=48, height=8, wrap=WORD)
		scroll = Scrollbar(frame, command=text.yview)
		text.grid(row=0, column=0, sticky=NSEW)
		scroll.grid(row=0, column=1, sticky=NS)
		text.insert('1.0', self.about_text)
		text.config(state=DISABLED, yscrollcommand=scroll.set)
		frame.grid(row=4, column=0, columnspan=2, pady=4)

	def button_box(self):
		frame = Frame(self)
		Button(frame, text=_("Close"), width=10, command=self.ok, default=ACTIVE).pack(
			side=LEFT, padx=2, pady=2)
		self.bind('<Return>', self.ok)
		frame.pack(side=RIGHT)

	def __OnHomePage(self, event=None):
		if self.home_page_command:
			self.home_page_command(interface)

# Initialization functions
root = None
interface = None
def init(app):
	global root, interface

	root = Tk()
	interface = Interface(app, root)
	return interface

def run():
	interface.post_init()

	root.mainloop()
	root.destroy()
