# optionsdialog.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Author: Brian Bernas
#
# The options dialog for the Tkinter interface

from __init__ import *
from Tkinter import *
from tkColorChooser import askcolor
from controls import Dialog
import tkMessageBox
import sys

_ = get_translation('tk_interface')

# The font for the section headers
section_font = None

# Store all of the options here
__options = {}

# Functions to access the options
def set_option(name, value):
	global __options

	__options[name] = value

def get_option(name, default=None):
	if __options.has_key(name):
		return __options[name]

	else:
		return default

#-------------------------------------------------------------------------------
class OptionsDlg(Dialog):
	def __init__(self, parent, title=None):
		Dialog.__init__(self, parent, title)
		self._resizable = TRUE

		global section_font

		# Variables
		self._parent = parent

		self._panes = [
			OptPaneGeneral,
			OptPaneStartup,
			OptPaneTaskList,
			OptPanePlugins,
			OptPaneAdvanced,
			]

		if sys.platform[:3] != 'win':
			self._panes.append(OptPaneWindowOffsets)

		self._curpane = None # The active option pane
		self.__cursel = None # The current selection for self._optionList

		# Get the default font
		font = parent.option_get('font', Tk)
		if font:
			f = font.split()
			f.append('bold')
			section_font = tuple(f)
		else:
			# Default back to Arial 10 pt.
			section_font = ('Arial', 10, 'bold')
			
	def body(self, master):
		# Set the default size of the dialog
		self.geometry('470x460')

		optionlist = self._optionList = Listbox(master, selectmode=SINGLE, width=15)
		optionlist.grid(row=0, sticky='nsw')

		frame = Frame(master, bd=2, relief=SUNKEN)
		font = (section_font[0], 14, 'bold')
		self._cursection = Label(frame, text="", font=font, anchor=E, bd=2, relief=RAISED)
		self._cursection.grid(row=0, column=0, sticky='new')
		self._pane = Frame(frame)
		self._pane.grid(row=1, column=0, ipady=5, sticky=NW)

		frame.grid_columnconfigure(0, weight=1)
		frame.grid_rowconfigure(1, weight=1)

		frame.grid(row=0, column=1, sticky=NSEW)

		master.grid_rowconfigure(0, weight=1)
		master.grid_columnconfigure(1, weight=1)

		# Insert the option group names
		for p in self._panes:
			optionlist.insert(END, p.name)

		optionlist.selection_set(0)
		self.__OnOptionList()

		optionlist.bind('<ButtonRelease-1>', self.__OnOptionList)

	def apply(self):
		# Save the settings of the current pane
		# If the proccess encounters an error, don't destroy the options dialog
		if not self._curpane.close():
			return FALSE

		return TRUE

	def __OnOptionList(self, event=None):
		sel = int(self._optionList.curselection()[0])

		if self.__cursel != sel:

			if self._curpane:
				if not self._curpane.close(): return
				self._curpane.destroy()

			self.__cursel = sel
			self._cursection['text'] = self._optionList.get(sel)

			optpane = self._panes[sel]

			# The arguments to pass to the OptPane* classes
			vars = [self._pane]

			if optpane == OptPaneWindowOffsets:
				vars.append(self._parent)

			if optpane and callable(optpane):
				self._curpane = tmp = apply(optpane, vars)
				tmp.pack(side=LEFT, fill=BOTH)

				tmp.load()
				tmp.body()

	# These access the global set_option and get_option functions
	def set_option(self, name, value):
		return set_option(name, value)

	def get_option(self, name, default=None):
		return get_option(name, default)

#-------------------------------------------------------------------------------
# Template for all option panes
class OptPane(Frame):
	def __init__(self, master):
		Frame.__init__(self, master)

	def _spacer(self, master):
		"""Create a empty frame for spacing out the sections"""
		Frame(master).grid(pady=5)

	def load(self):
		"""Initialize variables and get the options needed"""

	def close(self):
		"""Save the options out. Return true if the values could be saved successfully."""
		return TRUE

	def body(self):
		"""Create the controls here"""

	name = "None"

#-------------------------------------------------------------------------------
class OptPaneGeneral(OptPane):
	name = _("General")

	def load(self):
		self.save_win_pos = IntVar()
		self.save_win_size = IntVar()
		self.sort_filter = IntVar()
		self.usebrowser = IntVar()
		self.browser = StringVar()

		self.save_win_pos.set(get_option("savewinposition", TRUE))
		self.save_win_size.set(get_option("savewinsize", TRUE))
		self.sort_filter.set(get_option("sortandfilter", FALSE))
		self.browser.set(get_option("webbrowser", ''))

		if self.browser.get().strip():
			self.usebrowser.set(1)

	def close(self):
		set_option("savewinposition", self.save_win_pos.get())
		set_option("savewinsize", self.save_win_size.get())
		set_option("sortandfilter", self.sort_filter.get())

		if self.usebrowser.get():
			val = self.browser.get()
		else: val = ''

		set_option("webbrowser", val)

		return TRUE

	def body(self):
		Label(self, text=_("Window Settings"), font=section_font).grid(row=0, sticky=W)

		frame = Frame(self)
		Checkbutton(frame, text=_("Save window position"), variable=self.save_win_pos).grid(
			row=0, sticky=W)
		Checkbutton(frame, text=_("Save window size"), variable=self.save_win_size).grid(
			row=1, sticky=W)
		frame.grid(row=1, ipadx=15, sticky=W)

		self._spacer(self)

		Label(self, text=_("Sorting and Filter Settings"), font=section_font).grid(
			row=3, sticky=W)

		frame = Frame(self)
		Checkbutton(frame, text=_("Save sorting and filter modes"), variable=self.sort_filter).grid(
			row=0, sticky=W)
		frame.grid(row=4, ipadx=15, sticky=W)

		self._spacer(self)

		Label(self, text=_("Web Browser Settings"), font=section_font).grid(
			row=6, sticky=W)

		frame = Frame(self)
		Radiobutton(frame, text=_("Use default browser"), value=0, variable=self.usebrowser).grid(
			row=0, sticky=W)
		Radiobutton(frame, text=_("Enter manual comand"), value=1, variable=self.usebrowser).grid(
			row=1, sticky=W)
		frame2 = Frame(frame)
		Label(frame2, text=_("Command format: command \"%s\"")).grid(row=0, sticky=W)
		self._browserEntry = Entry(frame2, width=25, textvariable=self.browser)
		self._browserEntry.grid(row=1, sticky=W)
		frame2.grid(row=2, ipadx=20, sticky=W)
		frame.grid(row=7, ipadx=15, sticky=W)

		self.usebrowser.trace('w', self.__OnUseBrowser)
		self.__OnUseBrowser()

	def __OnUseBrowser(self, *args):
		if self.usebrowser.get():
			self._browserEntry['state'] = NORMAL
		else:
			self._browserEntry['state'] = DISABLED

#-------------------------------------------------------------------------------
class OptPaneStartup(OptPane):
	name = _("Startup")

	def load(self):
		self.load_file = IntVar()
		self.show_calendar = IntVar()
		self.show_groups = IntVar()

		self.load_file.set(get_option("startupfile", 0))
		self.show_calendar.set(get_option("showcalendar", FALSE))
		self.show_groups.set(get_option("showgroups", TRUE))

	def close(self):
		set_option("startupfile", self.load_file.get())
		set_option("showcalendar", self.show_calendar.get())
		set_option("showgroups", self.show_groups.get())

		return TRUE

	def body(self):
		Label(self, text=_("File Loaded On Startup"), font=section_font).grid(
			row=0, sticky=W)

		frame = Frame(self)
		Radiobutton(frame, text=_("Default File"), value=0, variable=self.load_file).grid(
			row=0, sticky=W)
		Radiobutton(frame, text=_("The file that was loaded last"), value=1,
			variable=self.load_file).grid(row=1, sticky=W)
		Radiobutton(frame, text=_("None"), value=2, variable=self.load_file).grid(row=2, sticky=W)
		frame.grid(row=1, ipadx=15, sticky=W)

		self._spacer(self)

		Label(self, text=_("Display"), font=section_font).grid(row=3, sticky=W)

		frame = Frame(self)
		Checkbutton(frame, text=_("Show calendar on startup"), variable=self.show_calendar).grid(
			row=0, sticky=W)
		Checkbutton(frame, text=_("Show groups on startup"), variable=self.show_groups).grid(row=1,
			sticky=W)
		frame.grid(row=4, ipadx=15, sticky=W)

#-------------------------------------------------------------------------------
class OptPaneTaskList(OptPane):
	name = _("Task List")

	def load(self):
		# Variables
		self.show_priority = IntVar()
		self.show_duedate = IntVar()
		self.show_finished = IntVar()
		self.show_finished_time = IntVar()

		# Listbox colors
		self._colors = {
			"itemfg": get_option("itemforeground"),
			"itemduefg": get_option("dueitemforeground"),
			"itemoverduefg": get_option("overdueitemforeground"),
			"itemselfg": get_option("selecteditemforeground"),
			"itemselduefg": get_option("selecteddueitemforeground"),
			"itemseloverduefg": get_option("selectedoverdueitemforeground"),
			"itembg": get_option("itembackground"),
			"itemselbg": get_option("selecteditembackground"),
			}

		self.show_priority.set(get_option("showpriority", TRUE))
		self.show_duedate.set(get_option("showduedate", FALSE))
		self.show_finished.set(get_option("showfinished", TRUE))
		self.show_finished_time.set(get_option("showfinishedtime", TRUE))

	def close(self):
		set_option("showpriority", self.show_priority.get())
		set_option("showduedate", self.show_duedate.get())
		set_option("showfinished", self.show_finished.get())
		set_option("showfinishedtime", self.show_finished_time.get())

		set_option("itemforeground", self._colors["itemfg"])
		set_option("dueitemforeground", self._colors["itemduefg"])
		set_option("overdueitemforeground", self._colors["itemoverduefg"])
		set_option("selecteditemforeground", self._colors["itemselfg"])
		set_option("selecteddueitemforeground", self._colors["itemselduefg"])
		set_option("selectedoverdueitemforeground", self._colors["itemseloverduefg"])
		set_option("itembackground", self._colors["itembg"])
		set_option("selecteditembackground", self._colors["itemselbg"])

		return TRUE

	def body(self):
		Label(self, text=_("Task List Columns"), font=section_font).grid(
			row=0, column=0, sticky=W)

		frame = Frame(self)
		Checkbutton(frame, text=_("Show Priority"), variable=self.show_priority).grid(
			row=1, sticky=W)
		Checkbutton(frame, text=_("Show Due Date"), variable=self.show_duedate).grid(
			row=2, sticky=W)
		Checkbutton(frame, text=_("Show Finished"), variable=self.show_finished).grid(
			row=3, sticky=W)
		Checkbutton(frame, text=_("Show Finished Time"), variable=self.show_finished_time).grid(
			row=4, sticky=W)
		frame.grid(row=1, ipadx=15, sticky=W)

		self._spacer(self)

		Label(self, text=_("Task Colors"), font=section_font).grid(row=3, sticky=W)

		frame = Frame(self)
		colorlist = Listbox(frame, bd=2, relief=SUNKEN, height=5)
		colorlist.grid(row=0, sticky=W)
		colorlist.insert(0, _("Item"), _("Due Item"), _("Over Due Item"))

		# Get the length of the last item and set the list's width to the same value
		# This isn't a smart hack, but it works
		colorlist['width'] = len(colorlist.get(2))

		self._swatchFrame = Frame(frame)
		Label(self._swatchFrame, justify=LEFT,
			text=_("Select an item to the left to\nchange its color settings.")).grid(
			row=0, sticky=W)
		self._swatchFrame.grid(row=0, column=1, sticky=W)

		frame.grid(row=4, ipadx=15, sticky=W)

		colorlist.bind('<ButtonRelease-1>', self.__OnColorListSelect)

	def __OnColorListSelect(self, event):
		sel = int(event.widget.curselection()[0])
		self.__draw_color_swatches(sel)

	def __OnColorSwatchClick(self, index, key):
		c = askcolor(self._colors[key])[1]
		if c: self._colors[key] = c

		# Redraw the color swatches
		self.__draw_color_swatches(index)

	def __draw_color_swatches(self, index):
		# Destroy them all. hah hah hah
		for c in self._swatchFrame.winfo_children():
			c.destroy()

		swatchlist = [(N_("Foreground"), "item%sfg" %("","due","overdue")[index]),
			(N_("Selected foreground"), "itemsel%sfg" %("","due","overdue")[index])]
		if index == 0:
			swatchlist.append((N_("Background"), "itembg"))
			swatchlist.append((N_("Selected background"), "itemselbg"))

		row = 0
		for text, i in swatchlist:
			canvas = Canvas(self._swatchFrame, bd=1, relief=SOLID, height=14, width=20,
				bg=self._colors[i])
			canvas.grid(row=row, column=0, sticky=W)
			Label(self._swatchFrame, text=_(text)).grid(row=row, column=1, sticky=W)

			canvas.bind('<Button-1>', lambda e, s=self, i=index, k=i: s.__OnColorSwatchClick(i, k))
			row = row + 1

#-------------------------------------------------------------------------------
class OptPanePlugins(OptPane):
	name = _("Plugins")

	def load(self):
		self.load_plugins = IntVar()

		self.load_plugins.set(get_option("loadplugins", 2))
		self.plugins = get_option("plugin_info", [])

	def close(self):
		set_option("loadplugins", self.load_plugins.get())

		return TRUE

	def body(self):
		Label(self, text=_("Loading Options"), font=section_font).grid(row=0, sticky=W)

		frame = Frame(self)
		Radiobutton(frame, text=_("Load all plugins"), value=2, variable=self.load_plugins).grid(
			row=0, sticky=W)
		Radiobutton(frame, text=_("Do not load plugins"), value=0, variable=self.load_plugins).grid(
			row=1, sticky=W)
		frame.grid(row=1, ipadx=15, sticky=W)

		Label(self, text=_("Plugin Information"), font=section_font).grid(row=2, sticky=W)

		frame = Frame(self)
		frame2 = Frame(frame)
		plglist = Listbox(frame2, bd=2, relief=SUNKEN, width=22, height=6)
		scroll = Scrollbar(frame2, command=plglist.yview)
		plglist.config(yscrollcommand=scroll.set)
		plglist.pack(side=LEFT)
		scroll.pack(side=LEFT, fill=Y)
		frame2.grid(row=0, sticky=W)
		self._infoText = text = Text(frame, bd=2, relief=RAISED, wrap=WORD, state=DISABLED,
			width=35, height=7)
		text.tag_configure('title', font=('arial', 12, 'bold'))
		text.tag_configure('section', font=('arial', 10, 'bold'))
		text.grid(row=1, sticky=W)

		frame.grid(row=3, ipadx=15, sticky=W)

		plglist.bind('<ButtonRelease-1>', self.__OnSelectPlugin)

		# Add all plugins to the list
		for p in self.plugins:
			plglist.insert(END, p['name'])

	def __OnSelectPlugin(self, event):
		cs = event.widget.curselection()
		if not cs:
			# If there is no selection, then just stop
			return

		i = int(cs[0])
		p = self.plugins[i]

		# Set the state to normal so text can be added
		self._infoText['state'] = NORMAL

		self._infoText.delete('1.0', END)
		self._infoText.insert(END, p['name'] + '\t', 'title', p['version'], 'section')
		self._infoText.insert(END, '\nAuthor: ', 'section', p['author'])
		if p['author_email']:
			self._infoText.insert(END, "\nAuthor's email: ", 'section', p['author_email'])
		if p['url']:
			self._infoText.insert(END, '\nURL: ', 'section', p['url'])
		if p['description']:
			self._infoText.insert(END, '\nDescription: ', 'section', p['description'])

		# Disable the control so the user doesn't play with it's contents
		self._infoText['state'] = DISABLED

#-------------------------------------------------------------------------------
class OptPaneWindowOffsets(OptPane):
	name = _("Window Offsets")

	def __init__(self, master, parent):
		Frame.__init__(self, master)

		self._parent_window = parent

	def load(self):
		self.xoffset = IntVar()
		self.yoffset = IntVar()

		self.xoffset.set(get_option("windowxoffset", 0))
		self.yoffset.set(get_option("windowyoffset", 0))

	def close(self):
		# Check for valid settings
		try:
			int(self.xoffset.get())
			int(self.yoffset.get())
		except ValueError:
			tkMessageBox.showerror(_("Invalid Value"),
				_("The offset values are currently invalid.\n"\
				"Only numerical values are permitted."))
			return FALSE

		set_option("windowxoffset", self.xoffset.get())
		set_option("windowyoffset", self.yoffset.get())

		return TRUE

	def body(self):
		Label(self, text=_("If you notice that the ToDo Manager window is not returning to "\
			"the same position that it was in when you last exited, then you need to adjust the "\
			"offset values for your window manager. Clicking the Configure button will attempt "\
			"to get the correct values."), justify=LEFT, wraplength=300).grid(
			row=0, pady=10, sticky=W)

		frame = Frame(self)
		self._xoffsetEntry = Entry(frame, width=3, textvariable=self.xoffset).grid(
			row=1, sticky=W)
		Label(frame, text=_("Horizontal Offset")).grid(row=1, column=1, sticky=W)
		self._yoffsetEntry = Entry(frame, width=3, textvariable=self.yoffset).grid(
			row=2, sticky=W)
		Label(frame, text=_("Vertical Offset")).grid(row=2, column=1, sticky=W)
		Button(frame, text=_("Configure"), command=self.__OnConfigure).grid(
			row=3, columnspan=2, sticky=W)
		frame.grid(row=1, ipadx=15, sticky=W)

	def __OnConfigure(self):
		parent = self._parent_window

		oldx = parent.winfo_x()
		oldy = parent.winfo_y()

		parent.geometry('+0+0')
		parent.update()

		xoffset = parent.winfo_x()
		yoffset = parent.winfo_y()
		self.xoffset.set(xoffset)
		self.yoffset.set(yoffset)

		parent.geometry('+%d+%d' %(oldx-xoffset, oldy-yoffset))

class OptPaneAdvanced(OptPane):
	name = _("Advanced")

	def load(self):
		self.format = StringVar()
		self.savebackup = IntVar()

		self.format.set(get_option("timeformat", ""))
		self.savebackup.set(get_option("savebackup", FALSE))

	def close(self):
		set_option("timeformat", self.format.get())
		set_option("savebackup", self.savebackup.get())

		return TRUE

	def body(self):
		Label(self, text=_("Date and Time Format"), font=section_font).grid(row=0, sticky=W)
		frame = Frame(self)
		Label(frame, text=_("Format:")).grid(row=0, column=0, sticky=W)
		entry = Entry(frame, width=22, textvariable=self.format).grid(row=0, column=1, sticky=W)
		frame.grid(row=1, ipadx=15, sticky=W)

		self._spacer(self)

		Label(self, text=_("File Options"), font=section_font).grid(row=3, sticky=W)
		frame = Frame(self)
		Checkbutton(frame, text=_("Save backup files (for crash recovery)"), justify=LEFT,
			variable=self.savebackup).grid(row=0, column=0, sticky=W)
		frame.grid(row=4, ipadx=15, sticky=W)
