# plugin.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Author: Brian Bernas
#
# Architecture for loading plugins into ToDo Manager

from __init__ import *
import os, imp, traceback
import tdmcalls

_ = get_translation('core')

plugin_module = None

# This holds all of the plugins that are loaded
_loaded_plugins = []

# The general plugin dictionary
plugin = {
	'name': None,
	'version': None,
	'description': '',
	'author': '',
	'author_email': '',
	'url': '',
	'type': None,
	'function': None,
	}

#-----------------------------------------------------------------------------
# Plugin exceptions
class PluginError(Exception):
	pass

#-----------------------------------------------------------------------------
# Plugin functions

def get_plugin_from_file(file):
	try:
		mod = imp.load_source(os.path.splitext(file)[0], file)
	except:
		traceback.print_exc()
		print _("Unable to load plugin from file: %s" %file)
		return None

	p = plugin.copy()

	try:
		mod.init(p)
		return p
	except AttributeError:
		return None

def get_all_plugins(path):
	plugins = []

	# If a string was passed, convert it to a list
	if type(path) == type(''):
		path = [path]

	# The path must be a list
	if type(path) != type([]):
		return None

	# Make sure the plugins are valid before trying to get files from them
	for p in path:
		if os.path.isdir(p):
			for f in os.listdir(p):
				# Get the full path to the file
				f = os.path.join(p, f)
				if os.path.splitext(f)[1] == '.py':
					plg = get_plugin_from_file(f)
					if plg: plugins.append(plg)

	return plugins

# This function is called to load all of the plugins from the plugins directory
def load_all_plugins(path):
	# If plugin_module equals None then don't load any plugins
	plugins = get_all_plugins(path)

	if plugins:
		for p in plugins:
			load_plugin(p)

def unload_all_plugins():
	global _loaded_plugins

	_loaded_plugins = []

def load_plugin(plugin):
	global _loaded_plugins

	if (not plugin['name']) or (not plugin['function']):
		return 0

	for plg in _loaded_plugins:
		if not plugin['type'] in ('file_import', 'file_export', 'task_sorting', 'task_filter'):
			raise PluginError, "The plugin type: \"%s\" is not valid." %type
			return 0

		# See if the plugin function is already in the list
		if plg['function'] == plugin['function']:
			raise PluginError, "The plugin \"%s\" is already loaded." %plg['name']
			return 0

	# Every plugin dictionary contains it's name, type, function, and loading value
	val = len(_loaded_plugins)
	plugin['value'] = val
	_loaded_plugins.append(plugin)

	return 1

def unload_plugin(val):
	global _loaded_plugins

	for plg in _loaded_plugins:
		if plg['value'] == val:
			_loaded_plugins.remove(plg)
			return 1

	return 0

# Create a blank task. This can be useful for import plugins
def create_blank_task():
	return {
		"Name": "",
		"Notes": "",
		"Finished": 0,
		"DisableFinish": 0,
		"FinishedDate": "",
		"DueDate": "",
		"Priority": 1,
		"Group": [],
		}
