#! /usr/bin/env python

# setup.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Author: Brian Bernas
#
# Python Distutils setup script for creating packages

"""
This file is provided for easy creation of distributions. It is not designed
to be used as a Python Distutils install file for this package, though
nothing restricts it from being used as one. This package does not need to be
installed to work properly.
"""

from distutils.core import setup
import time, os, sys

# Variables
_build = ""	# The build version
_oldver = 0

#-----------------------------------------------------------------------------
def set_main_version():
	"""Sets the version number by modifying __init__.py"""
	global _build, _oldver

	# Open the file and read it into memory
	file = open("__init__.py", 'r')
	lines = file.readlines()
	file.close()

	# Find the version variable
	i = 0
	for line in lines:
		if line[:9] == "version =":
			if not _build:
				_build = line[10:].strip()[1:-1]
			else: _oldver = 1
			break
		i = i + 1

	# Formating
	if _build[:3] == "CVS" and not _oldver:
		# Get the current time for the build Id.
		t = time.strftime("%Y%m%d", time.localtime(time.time()))
		# Changing the value
		lines[i] = "version = \"%s %s\"\n" %(_build, t)

	else:
		lines[i] = "version = \"%s\"\n" %_build

	# Write the file back out
	file = open("__init__.py", 'w')
	file.writelines(lines)
	file.close()

# Change the version number in main
set_main_version()
build = _build

# If this is a CVS branch, get the branch name
if build[:3] == "CVS":
	build = build.split()[-1].lower()

# Run all of the i18n scripts so translations are up to date
try:
	os.chdir("i18n/")
	os.system(sys.executable + " genpot.py")
	os.system(sys.executable + " genmo.py")
	os.chdir(os.pardir)
except IOError:
	print "There was an error while executing the i18n scripts."

# Run setup to create the distribution
setup(name="todo-manager",
	version=build,
	description="A simple task manager",
	long_description="A standard task manager with notes, due dates, priorities, multiple sorting"\
		" modes, task filters, and HTML exporting.  It also has a basic plugin architecture that"\
		" allows programmers to add additional sorting modes, filters, and file formats.",
	author="Brian Bernas",
	author_email="click81@softhome.net",
	url="http://todo-manager.sourceforge.net",
	platforms="Linux, Windows",
	license="Python",
	py_modules=[
		"controls",
		"interface",
		"main",
		"objectlistbox",
		"optionsdialog",
		"plugin",
		"tdmcalls",
		"tdm_calendar",
		"todo-manager"
		],
)

# Change the version back
set_main_version()
