# tdm_calendar.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Authors: Miles Wiehahn, Brian Bernas
#
# A custom calendar control
#
# $Revision: 1.17 $

from Tkinter import *
from calendar import monthrange, firstweekday
from controls import ThinButton
import time

class Calendar(Frame):
	def __init__(self, master, command=None, **kw):
		Frame.__init__(self, master)
		
		# To preserve some things
		if kw.has_key("background"):
			self.__bgcolor = kw['background']
		elif kw.has_key("bg"):
			self.__bgcolor = kw['bg']
		else: self.__bgcolor = None

		self.__seldate = None	# The date that is currently selected
		self.__LeftClickDateCommand = command

		# This holds any dates that have importance
		# Use add_important_date and remove_important_date to modify this
		self.__important_dates = []

		# Create the year and month selection buttons
		frame = Frame(self, bd=0)
		ThinButton(frame, text='<', command=lambda s=self: s.IncTime(-1)).pack(side=LEFT)
		self._yearLabel = Label(frame, width=10, bd=2, relief=RAISED)
		self._yearLabel.pack(side=LEFT)
		ThinButton(frame, text='>', command=lambda s=self: s.IncTime(1)).pack(side=LEFT)
		frame.grid(row=0, column=0, sticky=W)

		frame = Frame(self, bd=0)
		ThinButton(frame, text='<', command=lambda s=self: s.IncTime(0, -1)).pack(side=LEFT)
		self._monthLabel = Label(frame, width=10, bd=2, relief=RAISED)
		self._monthLabel.pack(side=LEFT)
		ThinButton(frame, text='>', command=lambda s=self: s.IncTime(0, 1)).pack(side=LEFT)
		frame.grid(row=0, column=1, sticky=E)

		self._calcanvas = Canvas(self, kw, height=165, width=205)

		# Create the day header
		days = range(7)

		c = 15
		for t in days:
			# Get the name of the day
			x = time.strftime('%a', (0, 0, 0, 0, 0, 0, t, 0, 0))
			self._calcanvas.create_text(c, 10, text=x, fill='blue')
			c = c + 30

		self._calcanvas.create_line(0, 20, 250, 20)

		self.SetCurrentDate()

		self._calcanvas.grid(row=1, column=0, columnspan=2, sticky=NSEW)

	def Draw(self):
		self._yearLabel['text'] = self.year
		self._monthLabel['text'] = time.strftime("%B", (self.year, self.month, self.day,
			0, 0, 0, 0, 0, 0))

		# Remove all of the current days
		self._calcanvas.delete('tmp')

		# monthrange returns first day of month and month length
		DayCount, DaysInMonth = monthrange(self.year, self.month)

		# So the calendar starts on the right day of the week
		if firstweekday() == 0:
			DayCount =- (DayCount)
		else:
			DayCount =- (DayCount + 1)

		for j in range(7):	# Row loop
			for i in range(7):	# Column loop
				DayCount = DayCount + 1

				if DayCount > 0 and DayCount <= DaysInMonth:
					fg = 'black'	# The default color

					if (self.year, self.month, DayCount) in self.__important_dates:
						# All important dates in blue
						fg = 'darkgreen'

					if (self.year, self.month, DayCount) == (self.year, self.month, self.day):
						fg = 'blue'

					if (self.year, self.month, DayCount) == self.__seldate:
						fg = 'darkred'	# The current day in red
						# Black border around the selected date
						self._calcanvas.create_rectangle((i*30)+5, (j*25)+20, (i*30)+25, (j*25)+40,
							fill='white', outline='black', tags='tmp')

					self._calcanvas.create_text((i*30)+15, (j*25)+30, text=str(DayCount), fill=fg,
						tags=('tmp', 'day'+`DayCount`))
					self._calcanvas.tag_bind('day'+`DayCount`, '<Button-1>',
						lambda e, s=self, p=DayCount: s.SelDate(s.year, s.month, p))

	# Sets the date that should be selected
	def SelDate(self, year, month, day):
		d = self.__check_date_values(year, month, day)
		self.__seldate = d

		if self.__LeftClickDateCommand:
			self.__LeftClickDateCommand(d)

		self.Draw()

	def SetDate(self, year, month, day):
		self.year, self.month, self.day = self.__check_date_values(year, month, day)

		self.Draw()

	def IncTime(self, year=0, month=0, day=0):
		self.year = self.year + year
		self.month = self.month + month
		self.day = self.day + day

		self.year, self.month, self.day = self.__check_date_values(self.year, self.month, self.day)
		self.Draw()

	# Return the current year, month and day of calendar
	def GetDate(self):
		return self.year,self.month,self.day

	# Return the date that has been selected
	def GetSelDate(self):
		return self.__seldate

	def SetCurrentDate(self):
		# Obtaining the current locale time
		temptime=time.localtime(time.time())

		self.year = temptime[0]
		self.month = temptime[1]
		self.day = temptime[2]

		self.__seldate = (self.year, self.month, self.day)

		self.Draw()

	def add_important_date(self, date):
		# date should be a tuple of (year, month, day)
		if len(date) > 2:
			# Just get the first 3 values and forget the rest
			dd = (date[0], date[1], date[2])

			self.__important_dates.append(dd)
			self.Draw()

			return TRUE

		else:
			return FALSE

	def remove_important_date(self, date):
		if date == ALL:
			# Clear the list
			del self.__important_dates
			self.__important_dates = []

		elif date in self.__important_dates:
			self.__important_dates.remove(date)

		else:
			return FALSE

		self.Draw()
		return TRUE

	def __check_date_values(self, year=None, month=None, day=None):
		if year == None:
			# Get the current year if one wasn't supplied
			tmp = time.localtime(time.time())
			year = tmp[0]

		if month != None:
			if month > 12 or month < 0:
				year = year + month/12
				month = abs(month) % 12
			elif month == 0:
				month = 12
				year = year - 1

		if day != None:
			i = monthrange(year, month)
			if day > i[1]:
				day = i[1]

		return (year, month, day)
