# tdmcalls.py
# ToDo Manager
# This software is released under the Python 2.0.1 license
#
# Author: Brian Bernas
#
# All kinds of little things in here

import os, sys, re
import time, calendar

def unicode_to_utf8(data):
	"""Convert unicode strings to utf-8 encoding"""

	# It must be a unicode string
	if type(data) == type(u''):
		data = data.encode('utf-8')

	return data

def get_root_directory():
	"""Returns the directory that the software is located in"""

	curdir = os.path.dirname(__file__)
	if curdir == os.curdir: curdir = ''
	dir = os.path.join(os.getcwd(), curdir)

	return dir

def get_user_directory():
	"""Returns the location of the users config directory"""

	userdir = None
	envar = None

	if os.environ.has_key('HOME'): envar = 'HOME'
	elif os.environ.has_key('USERPROFILE'): envar = 'USERPROFILE'

	# Get the location and name of the directory
	if envar:
		userdir = os.path.join(os.environ[envar],
			('.todo-manager', 'todo-manager')[sys.platform[:3] == 'win'])

	else:
		# Get the main directory
		userdir = get_root_directory()

	return userdir

# Date and time functions
def strptime(format, date_time):
	"""Parse a time string based on the format specified"""

	# Generate the regex that will be used to parse to date
	regex = re.compile("[/.,-]")

	ls = date_time.split()
	fs = format.split()

	dfmt = regex.split(fs[0])

	if ls:
		d = regex.split(ls[0])
		for i in range(len(d)):
			try:
				d[i] = int(d[i])
			except ValueError:
				d[i] = 0

		# Put 0 in the place of anything missing
		while len(d) < 3:
			d.append(0)

		if len(ls) > 1:
			t = ls[1].split(':')
			for i in range(len(t)):
				t[i] = int(t[i])

			# If the seconds weren't specified, add 0 in their spot
			while len(t) < 3:
				t.append(0)

			if len(ls) > 2:
				if (ls[2].lower() == "pm") and (int(t[0]) != 12):
					t[0] = int(t[0]) + 12
				# If the hour is 12 AM, change the numeric value to 0
				elif (ls[2].lower() == "am") and (int(t[0]) == 12):
					t[0] = 0
		else: t = [0, 0, 0]

		date = [0, 0, 0]
		i = 0
		for x in dfmt:
			if (x == '%y') or (x == '%Y'): a = 0
			elif x == '%m': a = 1
			elif x == '%d': a = 2

			date[a] = d[i]

			i = i + 1

		d = date

		# If the year that was passed was only 2 digits, then assume that the user
		# was referring to the current century
		if len(str(d[0])) < 4:
			curyear = str(time.localtime(time.time())[0])
			d[0] = int("%s%0.2d" %(curyear[:2], d[0]))

		# Get the weekday
		try:
			w = calendar.weekday(d[0], d[1], d[2])
		except OverflowError:
			w = 0

		# Format: (year, month, day, hour, minute, second, weekday, julian day, daylight savings)
		ptime = (d[0], d[1], d[2], t[0], t[1], t[2], w, 0, time.daylight)

		return tuple(ptime)

	return None

def check_time_values(t):
	"""Check the formatting and value limits of a time structure (a tuple with a length of 9)"""

	if not t: return None

	try:
		# Convert the tuple into a list so values can be changed
		lt = list(t)

		# The  list must contain all 9 values
		while len(lt) < 9:
			lt.append(0)
		
		# If the values go out of their permitted range, then change them
		if lt[1] > 12:
			lt[1] = 12

		monthlen = calendar.monthrange(lt[0], lt[1])
		if lt[2] > monthlen[1]:
			lt[2] = monthlen[1]

		# Time settings
		if lt[3] > 24:
			lt[3] = 0
		if lt[4] > 59:
			lt[4] = 59
		if lt[5] > 59:
			lt[5] = 59

		# Get the weekday
		lt[6] = calendar.weekday(lt[0], lt[1], lt[2])

		lt[8] = time.daylight	# Daylight savings flag

		return tuple(lt)

	except (IndexError, ValueError, TypeError, OverflowError):
		return None	# The format is invalid

def storetime(t):
	"""Convert a tuple to the internal time storage format (YYYY-MM-DD hh:mm:ss)"""

	x = check_time_values(t)

	if not x: return ''

	return time.strftime("%Y-%m-%d %H:%M:%S", x)

def curtime():
	"""Returns the current time"""

	return time.localtime(time.time())

def cmptime(time1, time2):
	"""Compare 2 time structures.
	Return values: 0 = equal, 1 = time1 > time 2, -1 = time1 < time 2."""

	# Return values: 0: same; 1: late; -1: early
	if time1 == time2: return 0
	elif time1 > time2: return -1
	else: return 1

	return -1

def cmpcurtime(t):
	"""Compare a time to the current time"""

	return cmptime(t, curtime())

def _parse_store_time(t):
	"""Parse a time string in the internal strorage format (YYYY-MM-DD hh:mm:ss)"""

	l = t.split()

	x = []
	if len(l):
		for i in l[0].split('-'):
			x.append(int(i))
	if len(l) > 1:
		for i in l[1].split(':'):
			x.append(int(i))

	return check_time_values(x)

def parse_time(t):
	"""Attempt to parse a time string that is either in the old format, or in the storage format"""

	try:
		if '-' in t:
			x = _parse_store_time(t)
		else:
			x = strptime("%m/%d/%Y", t)
		return x
	except:
		return None
